@@ -583,10 +583,10 @@ public:
         {
             if(!communicator->getDefaultRouter())
             {
-                Ice::RouterFinderPrxPtr finder =
-                    ICE_UNCHECKED_CAST(Ice::RouterFinderPrx, communicator->stringToProxy(_finder));
+                Ice::RouterFinderPrxPtr finder;
                 try
                 {
+                    finder = ICE_UNCHECKED_CAST(Ice::RouterFinderPrx, communicator->stringToProxy(_finder));
                     communicator->setDefaultRouter(finder->getRouter());
                 }
                 catch(const Ice::CommunicatorDestroyedException& ex)
