@@ -21,9 +21,10 @@ IceInternal::RegisterPluginsInit::RegisterPluginsInit()
     Ice::registerPluginFactory("IceTCP", createIceTCP, true);
 
     //
-    // Only include the UDP and WS transport plugins with non-static builds or Gem/PyPI builds.
+    // Only include the UDP and WS transport plugins with non-static builds or Gem/PyPI/Swift
+    // builds.
     //
-#if !defined(ICE_STATIC_LIBS) || defined(ICE_GEM) || defined(ICE_PYPI)
+#if !defined(ICE_STATIC_LIBS) || defined(ICE_GEM) || defined(ICE_PYPI) || defined(ICE_SWIFT)
     Ice::registerPluginFactory("IceUDP", createIceUDP, true);
     Ice::registerPluginFactory("IceWS", createIceWS, true);
 #endif
