@@ -110,6 +110,29 @@ class ThreadPoolDestroyedException
 {
 };
 
+#ifdef ICE_SWIFT
+string
+prefixToDispatchQueueLabel(const std::string& prefix)
+{
+    if(prefix == "Ice.ThreadPool.Client")
+    {
+        return "com.zeroc.ice.client";
+    }
+
+    if(prefix == "Ice.ThreadPool.Server")
+    {
+        return "com.zeroc.ice.server";
+    }
+
+    string::size_type end = prefix.find_last_of(".ThreadPool");
+    if(end == string::npos)
+    {
+        end = prefix.size();
+    }
+
+    return "com.zeroc.ice.oa." + prefix.substr(0, end);
+}
+#endif
 }
 
 Ice::DispatcherCall::~DispatcherCall()
@@ -245,7 +268,12 @@ IceInternal::ThreadPoolWorkQueue::getNativeInfo()
 
 IceInternal::ThreadPool::ThreadPool(const InstancePtr& instance, const string& prefix, int timeout) :
     _instance(instance),
+#ifdef ICE_SWIFT
+    _dispatchQueue(dispatch_queue_create(prefixToDispatchQueueLabel(prefix).c_str(),
+                                         DISPATCH_QUEUE_CONCURRENT_WITH_AUTORELEASE_POOL)),
+#else
     _dispatcher(_instance->initializationData().dispatcher),
+#endif
     _destroyed(false),
     _prefix(prefix),
     _selector(instance),
@@ -273,8 +301,20 @@ IceInternal::ThreadPool::ThreadPool(const InstancePtr& instance, const string& p
     SYSTEM_INFO sysInfo;
     GetSystemInfo(&sysInfo);
     int nProcessors = sysInfo.dwNumberOfProcessors;
+#   elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+    static int ncpu[2] = { CTL_HW, HW_NCPU };
+    size_t sz = sizeof(nProcessors);
+    int nProcessors;
+    if(sysctl(ncpu, 2, &nProcessors, &sz, 0, 0) == -1)
+    {
+        nProcessors = 1;
+    }
 #   else
     int nProcessors = static_cast<int>(sysconf(_SC_NPROCESSORS_ONLN));
+    if(nProcessors == -1)
+    {
+        nProcessors = 1;
+    }
 #   endif
 #endif
 
@@ -417,6 +457,9 @@ IceInternal::ThreadPool::ThreadPool(const InstancePtr& instance, const string& p
 IceInternal::ThreadPool::~ThreadPool()
 {
     assert(_destroyed);
+#ifdef ICE_SWIFT
+    dispatch_release(_dispatchQueue);
+#endif
 }
 
 void
@@ -531,6 +574,12 @@ IceInternal::ThreadPool::ready(const EventHandlerPtr& handler, SocketOperation o
 void
 IceInternal::ThreadPool::dispatchFromThisThread(const DispatchWorkItemPtr& workItem)
 {
+#ifdef ICE_SWIFT
+    dispatch_sync(_dispatchQueue, ^
+    {
+        workItem->run();
+    });
+#else
     if(_dispatcher)
     {
         try
@@ -566,6 +615,7 @@ IceInternal::ThreadPool::dispatchFromThisThread(const DispatchWorkItemPtr& workI
     {
         workItem->run();
     }
+#endif
 }
 
 void
@@ -603,6 +653,16 @@ IceInternal::ThreadPool::prefix() const
     return _prefix;
 }
 
+#ifdef ICE_SWIFT
+
+dispatch_queue_t
+IceInternal::ThreadPool::getDispatchQueue() const ICE_NOEXCEPT
+{
+    return _dispatchQueue;
+}
+
+#endif
+
 void
 IceInternal::ThreadPool::run(const EventHandlerThreadPtr& thread)
 {
@@ -619,6 +679,9 @@ IceInternal::ThreadPool::run(const EventHandlerThreadPtr& thread)
             }
             catch(const ThreadPoolDestroyedException&)
             {
+                Lock sync(*this);
+                --_inUse;
+                thread->setState(ICE_ENUM(ThreadState, ThreadStateIdle));
                 return;
             }
             catch(const exception& ex)
