@@ -5,6 +5,10 @@
 #ifndef ICE_STREAM_CONNECTOR_H
 #define ICE_STREAM_CONNECTOR_H
 
+#include <Ice/Config.h>
+
+#if TARGET_OS_IPHONE != 0
+
 #include <Ice/TransceiverF.h>
 #include <Ice/Connector.h>
 
@@ -44,3 +48,5 @@ private:
 }
 
 #endif
+
+#endif
