@@ -301,6 +301,12 @@ NodeSessionManager::activate()
     }
 }
 
+bool
+NodeSessionManager::isWaitingForCreate()
+{
+    return _thread->isWaitingForCreate();
+}
+
 bool
 NodeSessionManager::waitForCreate()
 {
