@@ -7,11 +7,21 @@
 
 #include <IceSSL/Plugin.h>
 #include <IceSSL/SSLEngineF.h>
+#include <IceSSL/ConnectionInfo.h>
 #include <Ice/CommunicatorF.h>
 
 namespace IceSSL
 {
 
+class ExtendedConnectionInfo : public ConnectionInfo
+{
+public:
+
+    TrustError errorCode;
+    std::string host;
+};
+ICE_DEFINE_PTR(ExtendedConnectionInfoPtr, ExtendedConnectionInfo);
+
 class ICESSL_API PluginI : public virtual IceSSL::Plugin
 {
 public:
