@@ -2089,14 +2089,14 @@ Slice::Gen::TypesVisitor::writeMemberHashCode(const DataMemberList& dataMembers,
                     }
                     default:
                     {
-                        _M << nl << "h_ = ((h_ << 5) + h_) ^ (2654435761u * " << name << ");";
+                        _M << nl << "h_ = ((h_ << 5) + h_) ^ (2654435761u * (uint)" << name << ");";
                         break;
                     }
                 }
             }
             else
             {
-                 _M << nl << "h_ = ((h_ << 5) + h_) ^ " << name << ";";
+                 _M << nl << "h_ = ((h_ << 5) + h_) ^ (uint)" << name << ";";
             }
         }
         else
