@@ -727,7 +727,9 @@ namespace Ice
             {
                 IceInternal.OutputStreamWrapper w = new IceInternal.OutputStreamWrapper(this);
                 IFormatter f = new BinaryFormatter();
+#pragma warning disable SYSLIB0011 // Type or member is obsolete
                 f.Serialize(w, o);
+#pragma warning restore SYSLIB0011 // Type or member is obsolete
                 w.Close();
             }
             catch(System.Exception ex)
