@@ -390,6 +390,11 @@ final class TransceiverI implements IceInternal.Transceiver
                     {
                         break;
                     }
+                    default: // 1.9 introduced NEEDS_UNWRAP_AGAIN for DTLS
+                    {
+                        assert(false);
+                        break;
+                    }
                     }
                     break;
                 }
@@ -414,6 +419,11 @@ final class TransceiverI implements IceInternal.Transceiver
                     status = result.getHandshakeStatus();
                     break;
                 }
+                default: // 1.9 introduced NEEDS_UNWRAP_AGAIN for DTLS
+                {
+                    assert(false);
+                    break;
+                }
                 }
 
                 if(result != null)
