@@ -95,6 +95,15 @@ public class AllTests
             java.util.Map<String, test.Ice.scope.Test.C> cmap3 = i.opCMap(cmap1, cmap2);
             test(cmap2.value.get("a").s.equals(s1));
             test(cmap3.get("a").s.equals(s1));
+
+            test.Ice.scope.Test.E1 e = i.opE1(test.Ice.scope.Test.E1.v1);
+            test(e == test.Ice.scope.Test.E1.v1);
+
+            test.Ice.scope.Test.S1 s = i.opS1(new test.Ice.scope.Test.S1("S1"));
+            test(s.s.equals("S1"));
+
+            test.Ice.scope.Test.C1 c = i.opC1(new test.Ice.scope.Test.C1("C1"));
+            test(c.s.equals("C1"));
         }
 
         {
@@ -138,6 +147,15 @@ public class AllTests
             java.util.Map<String, test.Ice.scope.Test.C> cmap3 = i.end_opCMap(cmap2, i.begin_opCMap(cmap1));
             test(cmap2.value.get("a").s.equals(s1));
             test(cmap3.get("a").s.equals(s1));
+
+            test.Ice.scope.Test.E1 e = i.end_opE1(i.begin_opE1(test.Ice.scope.Test.E1.v1));
+            test(e == test.Ice.scope.Test.E1.v1);
+
+            test.Ice.scope.Test.S1 s = i.end_opS1(i.begin_opS1(new test.Ice.scope.Test.S1("S1")));
+            test(s.s.equals("S1"));
+
+            test.Ice.scope.Test.C1 c = i.end_opC1(i.begin_opC1(new test.Ice.scope.Test.C1("C1")));
+            test(c.s.equals("C1"));
         }
 
         {
