@@ -9,7 +9,7 @@ def test(b):
     if not b:
         raise RuntimeError('test assertion failed')
 
-def twoways(helper, p):
+def twoways(helper, p, bprx):
 
     communicator = helper.communicator()
     literals = p.opStringLiterals()
@@ -1431,3 +1431,6 @@ def twoways(helper, p):
     p1 = { "test": "test" }
     (p3, p2) = p.opMDict2(p1)
     test(p3["test"] == "test" and p2["test"] == "test")
+
+    bprx.opB()
+    bprx.opIntf()
